"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BannerPlugin = void 0;
const zod_1 = require("zod");
const base_1 = require("./base");
const rule = zod_1.z.string().or(zod_1.z.instanceof(RegExp));
const rules = rule.or(rule.array());
const bannerFunction = zod_1.z
    .function()
    .args(zod_1.z.object({
    hash: zod_1.z.string(),
    chunk: zod_1.z.custom(),
    filename: zod_1.z.string()
}))
    .returns(zod_1.z.string());
const bannerContent = zod_1.z.string().or(bannerFunction);
const bannerPluginOptions = zod_1.z.strictObject({
    banner: bannerContent,
    entryOnly: zod_1.z.boolean().optional(),
    exclude: rules.optional(),
    include: rules.optional(),
    raw: zod_1.z.boolean().optional(),
    footer: zod_1.z.boolean().optional(),
    test: rules.optional()
});
const bannerPluginArgument = bannerContent.or(bannerPluginOptions);
function getRawBannerRule(condition) {
    if (typeof condition === "string") {
        return {
            type: "string",
            stringMatcher: condition
        };
    }
    if (condition instanceof RegExp) {
        return {
            type: "regexp",
            regexpMatcher: condition.source
        };
    }
    throw new Error("unreachable: condition should be one of string, RegExp");
}
function getRawBannerRules(condition) {
    if (!condition)
        return undefined;
    if (Array.isArray(condition)) {
        return {
            type: "array",
            arrayMatcher: condition.map(i => getRawBannerRule(i))
        };
    }
    return getRawBannerRule(condition);
}
function getRawBannerContent(content) {
    if (typeof content === "string") {
        return {
            type: "string",
            stringPayload: content
        };
    }
    if (typeof content === "function") {
        return {
            type: "function",
            fnPayload: content
        };
    }
    throw new Error("BannerContent should be a string or function");
}
exports.BannerPlugin = (0, base_1.create)(base_1.BuiltinPluginName.BannerPlugin, (args) => {
    if (typeof args === "string") {
        return {
            banner: getRawBannerContent(args)
        };
    }
    if (typeof args === "function") {
        return {
            banner: getRawBannerContent(args)
        };
    }
    return {
        banner: getRawBannerContent(args.banner),
        entryOnly: args.entryOnly,
        footer: args.footer,
        raw: args.raw,
        test: getRawBannerRules(args.test),
        include: getRawBannerRules(args.include),
        exclude: getRawBannerRules(args.exclude)
    };
});
