"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExternalsPlugin = void 0;
const base_1 = require("./base");
exports.ExternalsPlugin = (0, base_1.create)(base_1.BuiltinPluginName.ExternalsPlugin, (type, externals) => {
    return {
        type,
        externals: (Array.isArray(externals) ? externals : [externals]).map(getRawExternalItem)
    };
});
function getRawExternalItem(item) {
    if (typeof item === "string") {
        return { type: "string", stringPayload: item };
    }
    if (item instanceof RegExp) {
        return { type: "regexp", regexpPayload: item.source };
    }
    if (typeof item === "function") {
        return {
            type: "function",
            fnPayload: async (ctx) => {
                return await new Promise((resolve, reject) => {
                    const promise = item(ctx, (err, result, type) => {
                        if (err)
                            reject(err);
                        resolve({
                            result: getRawExternalItemValueFormFnResult(result),
                            external_type: type
                        });
                    });
                    if (promise && promise.then) {
                        promise.then(result => resolve({
                            result: getRawExternalItemValueFormFnResult(result),
                            external_type: undefined
                        }), e => reject(e));
                    }
                });
            }
        };
    }
    return {
        type: "object",
        objectPayload: Object.fromEntries(Object.entries(item).map(([k, v]) => [k, getRawExternalItemValue(v)]))
    };
}
function getRawExternalItemValueFormFnResult(result) {
    return result === undefined ? result : getRawExternalItemValue(result);
}
function getRawExternalItemValue(value) {
    if (typeof value === "string") {
        return { type: "string", stringPayload: value };
    }
    else if (typeof value === "boolean") {
        return { type: "bool", boolPayload: value };
    }
    else if (Array.isArray(value)) {
        return {
            type: "array",
            arrayPayload: value
        };
    }
    throw new Error("unreachable");
}
