"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HtmlRspackPlugin = void 0;
const zod_1 = require("zod");
const base_1 = require("./base");
const validate_1 = require("../util/validate");
const htmlRspackPluginOptions = zod_1.z.strictObject({
    filename: zod_1.z.string().optional(),
    template: zod_1.z.string().optional(),
    templateContent: zod_1.z.string().optional(),
    templateParameters: zod_1.z.record(zod_1.z.string()).optional(),
    inject: zod_1.z.enum(["head", "body"]).optional(),
    publicPath: zod_1.z.string().optional(),
    scriptLoading: zod_1.z.enum(["blocking", "defer", "module"]).optional(),
    chunks: zod_1.z.string().array().optional(),
    excludedChunks: zod_1.z.string().array().optional(),
    sri: zod_1.z.enum(["sha256", "sha384", "sha512"]).optional(),
    minify: zod_1.z.boolean().optional(),
    title: zod_1.z.string().optional(),
    favicon: zod_1.z.string().optional(),
    meta: zod_1.z.record(zod_1.z.string().or(zod_1.z.record(zod_1.z.string()))).optional()
});
exports.HtmlRspackPlugin = (0, base_1.create)(base_1.BuiltinPluginName.HtmlRspackPlugin, (c) => {
    (0, validate_1.validate)(c, htmlRspackPluginOptions);
    const meta = {};
    for (const key in c.meta) {
        const value = c.meta[key];
        if (typeof value === "string") {
            meta[key] = {
                name: key,
                content: value
            };
        }
    }
    return {
        ...c,
        meta
    };
});
