"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RspackOptionsApply = void 0;
const graceful_fs_1 = __importDefault(require("graceful-fs"));
const ResolveSwcPlugin_1 = require("./web/ResolveSwcPlugin");
const DefaultStatsFactoryPlugin_1 = require("./stats/DefaultStatsFactoryPlugin");
const DefaultStatsPrinterPlugin_1 = require("./stats/DefaultStatsPrinterPlugin");
const cleverMerge_1 = require("./util/cleverMerge");
const assert_1 = __importDefault(require("assert"));
const ignoreWarningsPlugin_1 = __importDefault(require("./lib/ignoreWarningsPlugin"));
class RspackOptionsApply {
    constructor() { }
    process(options, compiler) {
        var _a;
        (0, assert_1.default)(options.output.path, "options.output.path should have value after `applyRspackOptionsDefaults`");
        compiler.outputPath = options.output.path;
        compiler.name = options.name;
        compiler.outputFileSystem = graceful_fs_1.default;
        // if (options.externals) {
        // 	assert(
        // 		options.externalsType,
        // 		"options.externalsType should have value after `applyRspackOptionsDefaults`"
        // 	);
        // 	new ExternalsPlugin(options.externalsType, options.externals).apply(
        // 		compiler
        // 	);
        // }
        // if (options.externalsPresets.node) {
        // 	new NodeTargetPlugin().apply(compiler);
        // }
        // if (options.externalsPresets.electronMain) {
        // 	new ElectronTargetPlugin("main").apply(compiler);
        // }
        // if (options.externalsPresets.electronPreload) {
        // 	new ElectronTargetPlugin("preload").apply(compiler);
        // }
        // if (options.externalsPresets.electronRenderer) {
        // 	new ElectronTargetPlugin("renderer").apply(compiler);
        // }
        // if (
        // 	options.externalsPresets.electron &&
        // 	!options.externalsPresets.electronMain &&
        // 	!options.externalsPresets.electronPreload &&
        // 	!options.externalsPresets.electronRenderer
        // ) {
        // 	new ElectronTargetPlugin().apply(compiler);
        // }
        // if (
        // 	options.externalsPresets.web ||
        // 	(options.externalsPresets.node && options.experiments.css)
        // ) {
        // 	new HttpExternalsPlugin(!!options.experiments.css).apply(compiler);
        // }
        const runtimeChunk = options.optimization
            .runtimeChunk;
        if (runtimeChunk) {
            Object.entries(options.entry).forEach(([entryName, value]) => {
                if (value.runtime === undefined) {
                    value.runtime = runtimeChunk.name({ name: entryName });
                }
            });
        }
        // new EntryOptionPlugin().apply(compiler);
        (0, assert_1.default)(options.context, "options.context should have value after `applyRspackOptionsDefaults`");
        compiler.hooks.entryOption.call(options.context, options.entry);
        const { minimize, minimizer } = options.optimization;
        if (minimize && minimizer) {
            for (const item of minimizer) {
                if (typeof item === "function") {
                    item.call(compiler, compiler);
                }
                else if (item !== "...") {
                    item.apply(compiler);
                }
            }
        }
        if (options.builtins.devFriendlySplitChunks) {
            options.optimization.splitChunks = undefined;
        }
        if ((_a = options.devServer) === null || _a === void 0 ? void 0 : _a.hot) {
            options.output.strictModuleErrorHandling = true;
        }
        new ResolveSwcPlugin_1.ResolveSwcPlugin().apply(compiler);
        new DefaultStatsFactoryPlugin_1.DefaultStatsFactoryPlugin().apply(compiler);
        new DefaultStatsPrinterPlugin_1.DefaultStatsPrinterPlugin().apply(compiler);
        if (options.ignoreWarnings && options.ignoreWarnings.length > 0) {
            new ignoreWarningsPlugin_1.default(options.ignoreWarnings).apply(compiler);
        }
        compiler.hooks.afterPlugins.call(compiler);
        if (!compiler.inputFileSystem) {
            throw new Error("No input filesystem provided");
        }
        compiler.resolverFactory.hooks.resolveOptions
            .for("normal")
            .tap("RspackOptionsApply", resolveOptions => {
            resolveOptions = (0, cleverMerge_1.cleverMerge)(options.resolve, resolveOptions);
            resolveOptions.fileSystem = compiler.inputFileSystem;
            return resolveOptions;
        });
        compiler.resolverFactory.hooks.resolveOptions
            .for("context")
            .tap("RspackOptionsApply", resolveOptions => {
            resolveOptions = (0, cleverMerge_1.cleverMerge)(options.resolve, resolveOptions);
            resolveOptions.fileSystem = compiler.inputFileSystem;
            resolveOptions.resolveToContext = true;
            return resolveOptions;
        });
        compiler.hooks.afterResolvers.call(compiler);
    }
}
exports.RspackOptionsApply = RspackOptionsApply;
